import argparse

from pypisearch.search import Search


def main() -> None:
    """Main program entrypoint."""

    # Get command arguments
    arg_parser = argparse.ArgumentParser(
        description="Custom pip-search utility by pypi search line"
    )
    arg_parser.add_argument(
        "q", metavar="query", type=str, help="query for search"
    )
    arg_parser.add_argument(
        "-p",
        "--page",
        default=1,
        metavar="page",
        type=int,
        help="search page (default 1)",
    )
    args = arg_parser.parse_args()

    # Parse search url and print result table
    search = Search(query=args.q, page=args.page)
    print(search.tabulated_result)


if __name__ == "__main__":
    main()



from importlib.metadata import PackageNotFoundError
from importlib.metadata import version as pkg_version

import pypisearch.re_constants as const


class ResultItem:
    """Result item instance of search."""

    def __init__(self, plain_text: str) -> None:
        self.plain_text = plain_text

    @property
    def is_empty(self) -> bool:
        """Check is current instance empty."""

        return not all([self.name, self.version])

    @property
    def name(self) -> str:
        """Returns item name."""

        name = const.NAME_RE.findall(self.plain_text)
        return name[0] if name else ""

    @property
    def version(self) -> str:
        """Returns item version."""

        version = const.VERSION_RE.findall(self.plain_text)
        return version[0] if version else ""

    @property
    def description(self) -> str:
        """Returns item description."""

        description = const.DESCRIPTION_RE.findall(self.plain_text)
        return description[0] if description else ""

    @property
    def is_installed(self) -> bool:
        try:
            pkg_version(self.name)
        except PackageNotFoundError:
            return False
        else:
            return True

    @property
    def get_installed_version(self) -> str:
        return pkg_version(self.name) if self.is_installed else ""

    @property
    def installed_description(self) -> str:
        return (
            f"[installed {self.get_installed_version}] "
            if self.is_installed
            else ""
        )



import requests

import tabulate

from pypisearch import re_constants as const
from pypisearch.result_item import ResultItem


class Search:
    """Main search process instance."""

    def __init__(self, query: str, page: int = 1) -> None:
        self.search_url = f"https://pypi.org/search/?q={query}&page={page}"

    @property
    def get_page_data(self) -> str:
        """Returns page's HTML code."""

        return requests.get(url=self.search_url).text

    @property
    def plain_items(self) -> list:
        """Returns plain result items."""

        return const.ITEM_RE.split(self.get_page_data)

    @property
    def result(self) -> list:
        """Returns list of result instances."""

        return list(
            filter(
                lambda result_item: not result_item.is_empty,
                map(
                    lambda plain_item: ResultItem(plain_text=plain_item),
                    self.plain_items,
                ),
            )
        )

    @property
    def tabulated_result(self) -> str:
        """Returns tabulated list of results."""

        return (
            tabulate.tabulate(
                [
                    [
                        f"{item.name} ({item.version})",
                        f"{item.installed_description}{item.description}",
                    ]
                    for item in self.result
                ],
                tablefmt="plain",
            )
            or "Sorry, we haven't results by your query"
        )
